package com.perforce.hws.testing;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.testng.Assert;
import org.testng.annotations.Test;

import com.perforce.hws.util.ProcessHelpers;

import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;

/**
 * This is a test harness that is expected to be configured by a javascript test factory.
 *
 * (Don't include this by default into your testng.xml, the factory will be.)
 */
public class ExecJavascriptTest implements ProcessHelpers {

    private static final Logger logger = LoggerFactory.getLogger(ExecJavascriptTest.class);

    private String script;
    private String clientDir;

    public ExecJavascriptTest(String clientDir, String script) {
        this.clientDir = clientDir;
        this.script = script;
    }

    /**
     * Executes a single javascript test.
     *
     * @param script    The relative path from the root of the project, e.g., test/default_api_status_get.js
     * @param clientDir The absolute path of the root directory
     */
    @Test
    public void execJavascriptTest() throws IOException, InterruptedException {
        Path scriptPath = Paths.get(clientDir, script).toAbsolutePath();
        logger.info("Starting test {}", script);
        try {
            String output = execOutOrFail("node", scriptPath.toString());
            logger.debug("Results of test {}: \n{}", script, output);
        } catch (IllegalStateException e) {
            Assert.fail("Test " + script + " failed: " + e.getMessage());
        }
    }
}
