package com.perforce.hws.testing;

import com.perforce.hws.automation.ProjectLocations;
import com.perforce.hws.util.ProcessHelpers;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.testng.Assert;
import org.testng.annotations.Test;

import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;

/**
 * Used to execute a python test we've discovered in the 'tests' subdirectory.
 *
 * We use a bash wrapping script in order to execute in the virtual environment.
 */
public class ExecPythonTest implements ProcessHelpers, ProjectLocations {

    static final Logger logger = LoggerFactory.getLogger(ExecPythonTest.class);

    private String clientDir;
    private String script;

    public ExecPythonTest(String clientDir, String script) {
        this.clientDir = clientDir;
        this.script = script;
    }

    @Test
    public void execPythonTest() throws IOException, InterruptedException {
        Path scriptPath = Paths.get(clientDir, script).toAbsolutePath();
        logger.info("Running Python test {}", script);
        try {
            String output = execOutOrFailInDir(projectDir(), pythonCommand(), scriptPath.toString());
            logger.info("output of test {}:\n{}", script, output);
        } catch (IllegalStateException e) {
            logger.error("Python test {} failed:\n{}", script, e.getMessage());
            Assert.fail("Python test failed", e);
        }
    }

    public String projectDir() throws IOException {
        return Paths.get(projectRootOrFail(),
                "client-python")
                .toAbsolutePath().toString();
    }

    public String pythonCommand() throws IOException {
        return Paths.get(projectRootOrFail(),
                "client-python", "python.sh")
                .toAbsolutePath().toString();
    }
}
