package com.perforce.hws.testing.deployment.deb;

import com.perforce.hws.automation.DeploymentCleanup;
import com.perforce.hws.automation.NetworkUtil;
import com.perforce.hws.automation.ProjectLocations;
import com.perforce.hws.testing.deployment.ConfigHelpers;
import com.perforce.hws.util.TimingHelpers;
import com.perforce.hwsclient.ApiClient;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.testng.Assert;
import org.testng.annotations.Test;

import java.nio.file.Path;
import java.nio.file.Paths;

/**
 * Test out issues related to configuration during product upgrades.
 */
public class UpgradeConfigTests implements
        ConfigHelpers,
        DebDeployment,
        NetworkUtil,
        ProjectLocations,
        TimingHelpers {

    static final Logger logger = LoggerFactory.getLogger(UpgradeConfigTests.class);

    /**
     * This ensures that edits to the configuration file after installation are not overridden
     * by re-running the installation + configuration procedures again.
     */
    @Test
    public void upgradeDoesNotOverwriteConfig() throws Exception {
        DeploymentCleanup deploymentCleanup = new DeploymentCleanup();
        deploymentCleanup.run();

        installOldDebFile(baseDebUpgradePath());

        // Let's reset the serv er to be non-https.
        Path configPath = Paths.get("/", "opt", "perforce", "helix-web-services", "etc", "helix-web-services.conf").toAbsolutePath();
        logger.info("configPath {}", configPath);

        switchConfigToHttp(configPath);

        // Now that we've reconfigured, we need to restart
        execOrFail("service", "helix_web_services", "restart");

        boolean isRunning = waitFor(10, 1000, () -> !isPortAvailable(80));
        Assert.assertTrue(isRunning, "HWS isn't started at port 80");

        // Now that we've reconfigured, we need to restart
        execOrFail("service", "helix_web_services", "stop");

        upgradeDebBuild();

        isRunning = waitFor(10, 1000, () -> !isPortAvailable(80));
        Assert.assertTrue(isRunning, "HWS isn't resstarted at port 80");

        // Read the configuration file.
        assertConfigStillHttp(configPath);

        deploymentCleanup.run();
    }
}
