package com.perforce.hws.testing.deployment.rpm;

import com.perforce.hws.automation.DeploymentCleanup;
import com.perforce.hws.automation.NetworkUtil;
import com.perforce.hws.testing.deployment.ConfigHelpers;
import com.perforce.hws.util.TimingHelpers;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.testng.Assert;
import org.testng.annotations.Test;

/**
 * Created by tristan on 5/16/16.
 */
public class ReinstallTests implements ConfigHelpers, NetworkUtil, RpmDeployment,
        TimingHelpers {

    private static final Logger logger = LoggerFactory.getLogger(ReinstallTests.class);

    /**
     * Basically, make sure our post-install config stuff doesn't interrupt the flow of managing
     * things via a package manager.
     *
     * @throws Exception
     */
    @Test
    public void installUninstallInstallRpm() throws Exception {
        DeploymentCleanup deploymentCleanup = new DeploymentCleanup();
        deploymentCleanup.run();

        String rpmBuild = obtainRpmBuild();

        installRpmFile(rpmBuild);

        boolean isRunning = waitFor(10, 1000, () -> !isPortAvailable(443));
        Assert.assertTrue(isRunning, "HWS should be running after installation");

        logger.info("uninstall: {}", execOutOrFail("yum", "remove", "-y", "helix-web-services"));

        boolean isStopped = waitFor(10, 1000, () -> isPortAvailable(443));
        Assert.assertTrue(isStopped, "HWS should not be running after uninstallation");

        installRpmFile(rpmBuild);

        isRunning = waitFor(10, 1000, () -> !isPortAvailable(443));
        Assert.assertTrue(isRunning, "HWS is not running at 443");

        deploymentCleanup.run();
    }
}
