package com.perforce.hws.testing.p4base;

import com.perforce.hws.automation.AutomationServerProvider;
import com.perforce.hws.p4base.CloseableServer;
import com.perforce.hws.p4base.P4Methods;
import com.perforce.hws.p4base.ResultMap;
import com.perforce.hws.testing.P4TestMethods;
import com.perforce.hws.testing.ServerProvider;
import com.perforce.hws.util.StringUtils;
import com.perforce.p4java.exception.P4JavaException;
import org.testng.annotations.Test;

import java.net.URISyntaxException;
import java.util.List;

import static org.testng.Assert.*;

/**
 * Creates a simple class that can smoke-test the methods of P4Methods. <p/>
 */
public class P4MethodsTests implements P4Methods, P4TestMethods, StringUtils {

    @Test(dataProvider = "server", dataProviderClass = ServerProvider.class)
    public void exec(AutomationServerProvider provider) throws P4JavaException, URISyntaxException {
        try (CloseableServer server = obtainAsJDoe(provider)) {
            List<ResultMap> results = exec(server, "info");
            assertFalse(results.isEmpty());
            assertTrue(!results.get(0).get("serverVersion", String.class).isEmpty());
        }
    }

    @Test(dataProvider = "server", dataProviderClass = ServerProvider.class)
    public void execInputMap(AutomationServerProvider provider) throws P4JavaException, URISyntaxException {
        try (CloseableServer s = obtainAsSuper(provider)) {
            String name = randomString();

            List<ResultMap> results = exec(s, "user", "-o", name);
            results.get(0).put("Email", "testing@example.com");
            execInput(s, results.get(0), "user", "-i", "-f");

            List<ResultMap> loaded = exec(s, "user", "-o", name);
            assertEquals(results.get(0).get("Email", String.class),
                    loaded.get(0).get("Email", String.class));

            exec(s, "user", "-d", "-f", name);
        }
    }

    @Test(dataProvider = "server", dataProviderClass = ServerProvider.class)
    public void changePassword(AutomationServerProvider provider) throws P4JavaException, URISyntaxException {
        try (CloseableServer s = obtainAsJDoe(provider)) {
            changePassword(s, "johndoe1A!", "newpassword1A!");
            s.get().logout();

            s.get().login("newpassword1A!");

            changePassword(s, "newpassword1A!", "johndoe1A!");
        }
    }
}
