package com.perforce.hws.testing.server;

import com.perforce.hws.testing.ClientProvider;
import com.perforce.hws.util.StringUtils;
import com.perforce.hwsclient.DefaultApi;
import com.perforce.hwsclient.models.ClientCommand;
import com.perforce.hwsclient.models.ClientsCommand;

import org.testng.annotations.Test;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import static java.lang.String.format;
import static org.testng.Assert.*;

/**
 * Tests for methods under /p4/:api/clients
 */
public class PerforceClientsTests implements StringUtils {

    @Test(dataProvider = "ProjectClientAsJDoe",
            dataProviderClass = ClientProvider.class)
    public void clientCRUDLifecycle(DefaultApi api) throws IOException {
        String rand = randomString();

        String clientName = format("test-client-%s", rand);

        ClientCommand clientSpec = new ClientCommand();
        clientSpec.setClient(clientName);
        clientSpec.setHost(null);
        clientSpec.setAltRoots(Arrays.asList("/alt/1", "/alt/2"));

        api.addClient("localhost", clientSpec);

        List<ClientsCommand> clients =
                api.getClients("localhost");

        assertTrue(
                clients.stream().anyMatch(m -> m.getClient().equals(clientName))
        );


        ClientCommand saved = api.getClient("localhost", clientName);

        assertEquals(saved.getClient(), clientSpec.getClient());
        assertEquals(saved.getAltRoots(), Arrays.asList("/alt/1", "/alt/2"));

        ClientCommand toUpdate = new ClientCommand();
        toUpdate.setDescription(format("test %s", rand));

        api.updateClient("localhost", clientName, toUpdate);

        ClientCommand updated = api.getClient("localhost", clientName);

        assertEquals(updated.getDescription().trim(), toUpdate.getDescription().trim());

        api.deleteClient("localhost", clientName);

        List<ClientsCommand> clients2 =
                api.getClients("localhost");

        assertFalse(
                clients2.stream().anyMatch(m -> m.getClient().equals(api))
        );
    }
}

