package com.perforce.hws.testing.server;

import static org.testng.Assert.assertTrue;

import java.io.IOException;
import java.util.Arrays;

import org.testng.annotations.Test;

import com.perforce.hws.testing.ClientProvider;
import com.perforce.hwsclient.ApiClient.HwsRestException;
import com.perforce.hwsclient.DefaultApi;
import com.perforce.hwsclient.models.CommandRequest;
import com.perforce.hwsclient.models.CommandResponse;

/**
 * Tests /commands methods.
 */
public class PerforceCommandTests {

    @Test(dataProvider = "ProjectClientAsJDoe",
            dataProviderClass = ClientProvider.class)
    public void canCallInfoCommand(DefaultApi api) throws IOException {
        CommandResponse responses =
                api.runCommand("localhost", "info", null);

        assertTrue(
                responses.getResults().stream().anyMatch(m -> m.containsKey("serverVersion"))
        );
    }

    @Test(dataProvider = "ProjectClientAsJDoe",
            dataProviderClass = ClientProvider.class)
    public void canCallInfoCommandWithArgs(DefaultApi api) throws IOException {
        CommandResponse responses =
                api.runCommand("localhost", "files", Arrays.asList("-m", "10", "//depot/..."));

        assertTrue(
                responses.getResults().stream().anyMatch(m -> m.containsKey("depotFile"))
        );
    }


    @Test(dataProvider = "ProjectClientAsJDoe",
            dataProviderClass = ClientProvider.class,
            expectedExceptions = HwsRestException.class)
    public void failsOnMissingArg(DefaultApi api) {
      	api.runCommand("localhost", "files", Arrays.asList("-i"));
    }

    @Test(dataProvider = "ProjectClientAsJDoe",
            dataProviderClass = ClientProvider.class,
            expectedExceptions = HwsRestException.class)
    public void withInputFailsWithoutWhitelist(DefaultApi api) {
        CommandRequest commandRequest = new CommandRequest();
        commandRequest.put("Client", "test-client");

        api.runCommandWithInput("localhost", "client", null, commandRequest);
    }
}
