package com.perforce.hws.testing.server;

import com.perforce.hws.testing.ClientProvider;
import com.perforce.hws.util.StringUtils;
import com.perforce.hwsclient.DefaultApi;
import com.perforce.hwsclient.models.Counter;

import org.junit.Assert;
import org.testng.annotations.Test;

import java.io.IOException;
import java.util.List;

import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertTrue;
import static org.testng.AssertJUnit.assertEquals;

/**
 * Methods for /p4/:api/counters.
 */
public class PerforceCountersTests implements StringUtils {
    /**
     * Counter crud.
     *
     * @param api the api
     * @throws IOException Signals that an I/O exception has occurred.
     */
    @Test(dataProvider = "ProjectClientAsSuper",
            dataProviderClass = ClientProvider.class)
    public void counterCrud(final DefaultApi api) throws IOException {
        String rand = randomString();
        Counter counterSpec = new Counter();
        counterSpec.setValue("1");

        api.setCounter("localhost", rand, counterSpec);

        List<Counter> counters =
                api.getCounters("localhost");

        assertTrue(
            counters.stream().anyMatch(m -> m.getCounter().equals(rand)
            	&& m.getValue().equals("1"))
        );

        Counter saved =
                api.getCounter("localhost", rand);

        assertEquals(saved.getValue(), "1");

        api.incrementCounter("localhost", rand);

        Counter updated =
                api.getCounter("localhost", rand);

        assertEquals(updated.getValue(), "2");

        api.deleteCounter("localhost", rand);

        List<Counter> counters2 =
                api.getCounters("localhost");

        assertFalse(
                counters2.stream().anyMatch(m -> m.getCounter().equals(rand))
        );
    }
    
    /**
     * Test equals.
     */
    @Test
    public void testEquals() {
    	// Counters have a field the same as the class name
        // which was building the equals method incorrectly.
    	// This has been corrected in the template file for
    	// all classes so testing it here should verify the
    	// change.
    	Counter c1 = new Counter();
    	Counter c2 = new Counter();
    	c1.setCounter("counter");
    	c1.setValue("1");
    	c2.setCounter("counter");
    	c2.setValue("1");
    	Assert.assertTrue(c1.equals(c2) && c2.equals(c1));
    }
}
