package com.perforce.hws.testing.server;

import com.perforce.hws.testing.ClientProvider;
import com.perforce.hws.util.StringUtils;
import com.perforce.hwsclient.DefaultApi;
import com.perforce.hwsclient.models.StreamCommand;
import com.perforce.hwsclient.models.StreamsCommand;

import org.testng.annotations.Test;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import static java.lang.String.format;
import static org.testng.Assert.*;

/**
 * /p4/:api/streams/... tests
 */
public class PerforceStreamsTests implements StringUtils {

    @Test(dataProvider = "ProjectClientAsJDoe",
            dataProviderClass = ClientProvider.class)
    public void streamCRUDLifecycle(DefaultApi api) throws IOException {
        String rand = randomString();

        String streamName = format("test-stream-%s", rand);

        String streamPath = format("//my_project/%s", streamName);

        StreamCommand newStream = new StreamCommand();
        newStream.setStream(streamPath);
        newStream.setName(streamName);
        newStream.setDescription(format("test", rand));
        newStream.setType("mainline");
        newStream.setParent("none");
        newStream.setPaths(Arrays.asList("share ..."));
        newStream.setOwner("jdoe");

        api.addStream("localhost", newStream);

        List<StreamsCommand> allStreams =
                api.getStreams("localhost");

        assertTrue(
                allStreams.stream().anyMatch(m -> m.getStream().equals(streamPath))
        );

        StreamCommand saved =
                api.getStream("localhost", streamPath);

        assertEquals(saved.getName(), newStream.getName());

        StreamCommand toUpdate = new StreamCommand();
        toUpdate.setDescription(format("update %s", rand));

        api.updateStream("localhost", streamPath, toUpdate);

        StreamCommand updated =
                api.getStream("localhost", streamPath);

        assertEquals(updated.getDescription().toString().trim(),
                toUpdate.getDescription().toString().trim());

        api.deleteStream("localhost", streamPath);

        List<StreamsCommand> allStreams2 =
                api.getStreams("localhost");

        assertFalse(
                allStreams2.stream().anyMatch(m -> m.getStream().equals(streamPath))
        );
    }
}
