package com.perforce.hws.testing.server.git_fusion;

import com.perforce.hws.p4base.ResultMap;
import com.perforce.hws.testing.ClientProvider;
import com.perforce.hws.util.StringUtils;
import com.perforce.hwsclient.AlphaApi;
import com.perforce.hwsclient.DefaultApi;
import com.perforce.hwsclient.models.CommandResponse;
import com.perforce.hwsclient.models.GitFusionRepoBranchConfig;
import com.perforce.hwsclient.models.GitFusionRepoConfig;
import com.perforce.hwsclient.models.GitFusionRepoId;

import org.testng.annotations.Test;

import java.util.Arrays;
import java.util.List;

import static org.testng.Assert.*;

/**
 * Tests methods related to /git-fusion-repos...
 */
public class GitFusionRepoConfigTests implements StringUtils {

    /**
     * A basic test to ensure we can just do simple CRUD operations on a repo with a single branch.
     *
     * We set a lot of values here not to test validity, but just to test we can actually transfer
     * data.
     */
    @Test(dataProvider = "ProjectClientAsJDoeWithAlpha",
            dataProviderClass = ClientProvider.class,
            groups = "no-cloud")
    public void testGitFusionRepoCRUD(DefaultApi defaultApi, AlphaApi alphaApi) {
        String rand = randomString();
        String repoName = "group/repo-" + rand;

        GitFusionRepoConfig repoConfig = new GitFusionRepoConfig();
        repoConfig.setName(repoName);
        repoConfig.setDescription("description " + rand);

        GitFusionRepoBranchConfig master = new GitFusionRepoBranchConfig();
        master.setGitBranchName("master");
        master.setGitBranchId("master");
        master.setView(Arrays.asList(String.format("//depot/main/%s/... ...", repoName)));
        repoConfig.setBranches(Arrays.asList(master));

        alphaApi.addGitFusionRepo("localhost", repoConfig);

        List<GitFusionRepoId> repoIds = alphaApi.getGitFusionRepos("localhost");

        String repoId = repoIds.stream().filter(id -> id.getName().equals(repoName))
                .map(id -> id.getId())
                .findFirst().orElse(null);

        assertNotNull(repoId);

        GitFusionRepoConfig savedConfig = alphaApi.getGitFusionRepo("localhost", repoId);

        assertEquals(savedConfig.getDescription(), repoConfig.getDescription());
        assertEquals(savedConfig.getBranches(), repoConfig.getBranches());

        alphaApi.deleteGitFusionRepo("localhost", repoId);

        repoIds = alphaApi.getGitFusionRepos("localhost");
        assertFalse(repoIds.stream().anyMatch(id -> id.getId().equals(repoId)));
    }
}
